@@ -2,6 +2,7 @@ package runner
 
 import (
 	"github.com/projectdiscovery/gologger"
+	updateutils "github.com/projectdiscovery/utils/update"
 )
 
 const banner = `
@@ -9,14 +10,22 @@ const banner = `
    ___  _______ __ __ (_) _/_ __
   / _ \/ __/ _ \\ \ // / _/ // /
  / .__/_/  \___/_\_\/_/_/ \_, / 
-/_/                      /___/	v0.0.8
+/_/                      /___/
 `
 
 // Version is the current version
-const Version = `0.0.8`
+const version = `0.0.9`
 
 // showBanner is used to show the banner to the user
 func showBanner() {
 	gologger.Print().Msgf("%s\n", banner)
 	gologger.Print().Msgf("\t\tprojectdiscovery.io\n\n")
 }
+
+// GetUpdateCallback returns a callback function that updates proxify
+func GetUpdateCallback() func() {
+	return func() {
+		showBanner()
+		updateutils.GetUpdateToolCallback("proxify", version)()
+	}
+}
