@@ -3,9 +3,10 @@ package file
 import (
 	"fmt"
 	"os"
-	"path"
+	"path/filepath"
 
 	"github.com/projectdiscovery/proxify/pkg/types"
+	fileutil "github.com/projectdiscovery/utils/file"
 )
 
 // Options required for file export
@@ -21,17 +22,14 @@ type Client struct {
 
 // New creates and returns a new client for file based logging
 func New(option *Options) (*Client, error) {
-	return &Client{
-		options: &Options{
-			OutputFolder: option.OutputFolder,
-		},
-	}, CreateOutputFolder(option.OutputFolder)
+	client := &Client{options: &Options{OutputFolder: option.OutputFolder}}
+	return client, fileutil.CreateFolder(option.OutputFolder)
 }
 
 // Store writes the log to the file
 func (c *Client) Save(data types.OutputData) error {
 	// generate the file destination file name
-	destFile := path.Join(c.options.OutputFolder, fmt.Sprintf("%s.%s", data.Name, "txt"))
+	destFile := filepath.Join(c.options.OutputFolder, fmt.Sprintf("%s.%s", data.Name, "txt"))
 	// if it's a response and file doesn't exist skip
 	f, err := os.OpenFile(destFile, os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
 	if err != nil {
@@ -41,8 +39,3 @@ func (c *Client) Save(data types.OutputData) error {
 	fmt.Fprint(f, data.DataString)
 	return f.Close()
 }
-
-// CreateOutputFolder creates the output folder if it doesn't exist
-func CreateOutputFolder(outputFolder string) error {
-	return os.MkdirAll(outputFolder, 0755)
-}
